"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updatePackageJson = void 0;
const path_1 = require("path");
const builder_1 = require("@build-script/builder");
const node_json_edit_1 = require("@idlebox/node-json-edit");
const unipm_1 = require("unipm");
const argParse_1 = require("../inc/argParse");
const paths_1 = require("../inc/paths");
const dependenciesFields = [
    'dependencies',
    'devDependencies',
    'optionalDependencies',
    'peerDependencies',
    'bundledDependencies',
];
const packageJsonSort = [
    'workspaces',
    'name',
    'type',
    'version',
    'private',
    'description',
    'keywords',
    'author',
    'homepage',
    'bugs',
    'license',
    'contributors',
    'repository',
    'engines',
    'os',
    'cpu',
    'bin',
    'main',
    'flow:main',
    'jsnext:main',
    'module',
    'esnext',
    'es2015',
    'esm',
    'unpkg',
    'browser',
    'module-browser',
    'browserslist',
    'react-native',
    'typings',
    'types',
    'man',
    'sideEffects',
    'source',
    'umd:main',
    'exports',
    'scripts',
    'config',
    ...dependenciesFields,
    'files',
    'publishConfig',
    'monorepo',
    'flat',
    'resolutions',
    'bolt',
    'jspm',
];
async function updatePackageJson({ isESNext, hookMode, dualPackage, outDir }) {
    const configRel = (0, paths_1.relativePosix)(argParse_1.PROJECT_ROOT, argParse_1.CONFIG_FILE);
    const packageJson = await (0, node_json_edit_1.loadJsonFile)((0, path_1.resolve)(argParse_1.PROJECT_ROOT, 'package.json'));
    const relOut = './' + path_1.posix.relative(argParse_1.PROJECT_ROOT, outDir + '/' + paths_1.INDEX_FILE_NAME);
    if (isESNext) {
        packageJson.type = 'module';
        if (dualPackage) {
            console.log('inserting dual package.');
            packageJson.main = relOut + '.cjs';
            packageJson.module = relOut + '.js';
            if (!packageJson.exports) {
                packageJson.exports = {};
            }
            if (!packageJson.exports['.']) {
                packageJson.exports['.'] = {};
            }
            packageJson.exports['.']['require'] = relOut + '.cjs';
            packageJson.exports['.']['import'] = relOut + '.js';
        }
        else {
            packageJson.main = relOut + '.js';
        }
    }
    else {
        packageJson.type = 'commonjs';
        packageJson.main = relOut + '.js';
    }
    packageJson.typings = 'docs/package-public.d.ts';
    if (hookMode) {
        const myFile = '@build-script/export-all-in-one/build-script-register';
        const previous = (await (0, builder_1.getPlugin)(myFile)) || [];
        await (0, builder_1.registerPlugin)(myFile, [dualPackage ? '' : argParse_1.NO_DUAL_FLAG, ...previous, configRel].filter(uniqNotEmpty));
    }
    else {
        if (!packageJson.scripts) {
            packageJson.scripts = {};
        }
        if (!packageJson.scripts['build:export-all-in-one']) {
            packageJson.scripts['export-all-in-one'] =
                'export-all-in-one ' + (dualPackage ? argParse_1.NO_DUAL_FLAG + ' ' : '') + configRel;
        }
    }
    if (!packageJson.devDependencies || !packageJson.devDependencies['@build-script/export-all-in-one']) {
        const oldFormat = (0, node_json_edit_1.getFormatInfo)(await (0, node_json_edit_1.loadJsonFile)((0, path_1.resolve)(argParse_1.PROJECT_ROOT, 'package.json')));
        const pm = await (0, unipm_1.getPackageManager)({ cwd: argParse_1.PROJECT_ROOT });
        await pm.install('--dev', '@build-script/export-all-in-one');
        const packageJson = require((0, path_1.resolve)(argParse_1.PROJECT_ROOT, 'package.json'));
        (0, node_json_edit_1.reformatJson)(packageJson, oldFormat);
    }
    for (const item of dependenciesFields) {
        sortAlphabet(packageJson, item);
    }
    (0, node_json_edit_1.sortObjectWithTargetOrder)(packageJson, packageJsonSort);
    await (0, node_json_edit_1.writeJsonFileBack)(packageJson);
}
exports.updatePackageJson = updatePackageJson;
function uniqNotEmpty(item, index, self) {
    return item && index === self.lastIndexOf(item);
}
function sortAlphabet(packageJson, key) {
    if (!packageJson[key])
        return;
    const keys = Object.keys(packageJson[key]).sort(strOrder);
    const ret = {};
    for (const item of keys) {
        ret[item] = packageJson[key][item];
    }
    packageJson[key] = ret;
}
function strOrder(a, b) {
    if (a > b)
        return 1;
    if (a < b)
        return -1;
    return 0;
}
//# sourceMappingURL=updatePackageJson.js.map